@@ -1,17 +1,9 @@
 use binread::NullString;
-use byteorder::{LittleEndian, WriteBytesExt};
 use std::io::{Cursor, Seek, SeekFrom, Write};
 
 use crate::{
-    anim::*,
-    formats::{
-        mesh::*,
-        nrpd::{FrameBuffer, NrpdState, RenderPassDataType},
-    },
-    matl::*,
-    shdr::*,
-    skel::*,
-    CString, RelPtr64, SsbhArray, SsbhByteBuffer, SsbhFile, SsbhWrite,
+    anim::*, formats::mesh::*, matl::*, shdr::*, skel::*, CString, Ptr64, RelPtr64, SsbhArray,
+    SsbhByteBuffer, SsbhFile, SsbhWrite,
 };
 
 fn round_up(value: u64, n: u64) -> u64 {
@@ -19,16 +11,24 @@ fn round_up(value: u64, n: u64) -> u64 {
     ((value + n - 1) / n) * n
 }
 
+fn write_u64<W: Write + Seek>(writer: &mut W, value: u64) -> std::io::Result<()> {
+    writer.write_all(&value.to_le_bytes())
+}
+
+fn write_u32<W: Write + Seek>(writer: &mut W, value: u32) -> std::io::Result<()> {
+    writer.write_all(&value.to_le_bytes())
+}
+
 fn write_relative_offset<W: Write + Seek>(writer: &mut W, data_ptr: &u64) -> std::io::Result<()> {
     let current_pos = writer.stream_position()?;
-    writer.write_u64::<LittleEndian>(*data_ptr - current_pos)?;
+    write_u64(writer, *data_ptr - current_pos)?;
     Ok(())
 }
 
 macro_rules! ssbh_write_c_enum_impl {
     ($enum_type:ident,$underlying_type:ident) => {
         impl SsbhWrite for $enum_type {
-            fn write_ssbh<W: std::io::Write + std::io::Seek>(
+            fn ssbh_write<W: std::io::Write + std::io::Seek>(
                 &self,
                 writer: &mut W,
                 _data_ptr: &mut u64,
@@ -55,9 +55,9 @@ ssbh_write_c_enum_impl!(TrackType, u8);
 ssbh_write_c_enum_impl!(AnimType, u64);
 
 ssbh_write_c_enum_impl!(AttributeDataTypeV8, u32);
-ssbh_write_c_enum_impl!(AttributeDataType, u32);
+ssbh_write_c_enum_impl!(AttributeDataTypeV10, u32);
 ssbh_write_c_enum_impl!(AttributeUsageV8, u32);
-ssbh_write_c_enum_impl!(AttributeUsageV10, u32);
+ssbh_write_c_enum_impl!(AttributeUsageV9, u32);
 ssbh_write_c_enum_impl!(RiggingType, u32);
 ssbh_write_c_enum_impl!(DrawElementType, u32);
 
@@ -70,15 +70,13 @@ ssbh_write_c_enum_impl!(MagFilter, u32);
 ssbh_write_c_enum_impl!(FilteringType, u32);
 ssbh_write_c_enum_impl!(ParamId, u64);
 
-ssbh_write_c_enum_impl!(RenderPassDataType, u64);
-
 ssbh_write_c_enum_impl!(BillboardType, u8);
 
 macro_rules! ssbh_write_impl {
     ($($id:ident),*) => {
         $(
             impl SsbhWrite for $id {
-                fn write_ssbh<W: std::io::Write + std::io::Seek>(
+                fn ssbh_write<W: std::io::Write + std::io::Seek>(
                     &self,
                     writer: &mut W,
                     _data_ptr: &mut u64,
@@ -97,132 +95,14 @@ macro_rules! ssbh_write_impl {
 
 ssbh_write_impl!(u8, u16, u32, u64, u128, i8, i16, i32, i64, i128, f32, f64);
 
-impl SsbhWrite for VertexWeights {
-    fn write_ssbh<W: std::io::Write + std::io::Seek>(
-        &self,
-        writer: &mut W,
-        data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        match self {
-            VertexWeights::VertexWeightsV8(v) => v.write_ssbh(writer, data_ptr),
-            VertexWeights::VertexWeightsV10(v) => v.write_ssbh(writer, data_ptr),
-        }
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        match self {
-            VertexWeights::VertexWeightsV8(v) => v.size_in_bytes(),
-            VertexWeights::VertexWeightsV10(v) => v.size_in_bytes(),
-        }
-    }
-}
-
-impl SsbhWrite for MeshAttributes {
-    fn write_ssbh<W: std::io::Write + std::io::Seek>(
-        &self,
-        writer: &mut W,
-        data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        match self {
-            MeshAttributes::AttributesV8(v) => v.write_ssbh(writer, data_ptr),
-            MeshAttributes::AttributesV10(v) => v.write_ssbh(writer, data_ptr),
-        }
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        match self {
-            MeshAttributes::AttributesV8(v) => v.size_in_bytes(),
-            MeshAttributes::AttributesV10(v) => v.size_in_bytes(),
-        }
-    }
-}
-
-impl SsbhWrite for NrpdState {
-    fn write_ssbh<W: std::io::Write + std::io::Seek>(
-        &self,
-        writer: &mut W,
-        data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        match self {
-            NrpdState::Sampler(v) => v.write_ssbh(writer, data_ptr),
-            NrpdState::RasterizerState(v) => v.write_ssbh(writer, data_ptr),
-            NrpdState::DepthState(v) => v.write_ssbh(writer, data_ptr),
-            NrpdState::BlendState(v) => v.write_ssbh(writer, data_ptr),
-        }
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        match self {
-            NrpdState::Sampler(v) => v.size_in_bytes(),
-            NrpdState::RasterizerState(v) => v.size_in_bytes(),
-            NrpdState::DepthState(v) => v.size_in_bytes(),
-            NrpdState::BlendState(v) => v.size_in_bytes(),
-        }
-    }
-}
-
-impl SsbhWrite for FrameBuffer {
-    fn write_ssbh<W: std::io::Write + std::io::Seek>(
-        &self,
-        writer: &mut W,
-        data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        match self {
-            FrameBuffer::Framebuffer0(v) => v.write_ssbh(writer, data_ptr),
-            FrameBuffer::Framebuffer1(v) => v.write_ssbh(writer, data_ptr),
-            FrameBuffer::Framebuffer2(v) => v.write_ssbh(writer, data_ptr),
-        }
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        match self {
-            FrameBuffer::Framebuffer0(v) => v.size_in_bytes(),
-            FrameBuffer::Framebuffer1(v) => v.size_in_bytes(),
-            FrameBuffer::Framebuffer2(v) => v.size_in_bytes(),
-        }
-    }
-}
-
-impl SsbhWrite for Param {
-    fn write_ssbh<W: std::io::Write + std::io::Seek>(
-        &self,
-        writer: &mut W,
-        data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        match self {
-            Param::Float(v) => v.write_ssbh(writer, data_ptr),
-            Param::Boolean(v) => v.write_ssbh(writer, data_ptr),
-            Param::Vector4(v) => v.write_ssbh(writer, data_ptr),
-            Param::MatlString(v) => v.write_ssbh(writer, data_ptr),
-            Param::Sampler(v) => v.write_ssbh(writer, data_ptr),
-            Param::UvTransform(v) => v.write_ssbh(writer, data_ptr),
-            Param::BlendState(v) => v.write_ssbh(writer, data_ptr),
-            Param::RasterizerState(v) => v.write_ssbh(writer, data_ptr),
-        }
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        match self {
-            Param::Float(v) => v.size_in_bytes(),
-            Param::Boolean(v) => v.size_in_bytes(),
-            Param::Vector4(v) => v.size_in_bytes(),
-            Param::MatlString(v) => v.size_in_bytes(),
-            Param::Sampler(v) => v.size_in_bytes(),
-            Param::UvTransform(v) => v.size_in_bytes(),
-            Param::BlendState(v) => v.size_in_bytes(),
-            Param::RasterizerState(v) => v.size_in_bytes(),
-        }
-    }
-}
-
 impl<T: binread::BinRead + SsbhWrite> SsbhWrite for Option<T> {
-    fn write_ssbh<W: Write + Seek>(
+    fn ssbh_write<W: Write + Seek>(
         &self,
         writer: &mut W,
         data_ptr: &mut u64,
     ) -> std::io::Result<()> {
         match self {
-            Some(value) => value.write_ssbh(writer, data_ptr),
+            Some(value) => value.ssbh_write(writer, data_ptr),
             None => Ok(()),
         }
     }
@@ -256,17 +136,17 @@ fn write_array_header<W: Write + Seek>(
 
     // Don't write the offset for empty arrays.
     if count == 0 {
-        writer.write_u64::<LittleEndian>(0u64)?;
+        write_u64(writer, 0u64)?;
     } else {
         write_relative_offset(writer, &data_ptr)?;
     }
 
-    writer.write_u64::<LittleEndian>(count as u64)?;
+    write_u64(writer, count as u64)?;
     Ok(())
 }
 
 impl SsbhWrite for SsbhByteBuffer {
-    fn write_ssbh<W: Write + Seek>(
+    fn ssbh_write<W: Write + Seek>(
         &self,
         writer: &mut W,
         data_ptr: &mut u64,
@@ -295,7 +175,7 @@ impl SsbhWrite for SsbhByteBuffer {
 }
 
 impl<T: SsbhWrite + binread::BinRead> SsbhWrite for &[T] {
-    fn write_ssbh<W: Write + Seek>(
+    fn ssbh_write<W: Write + Seek>(
         &self,
         writer: &mut W,
         data_ptr: &mut u64,
@@ -307,7 +187,7 @@ impl<T: SsbhWrite + binread::BinRead> SsbhWrite for &[T] {
         }
 
         for element in self.iter() {
-            element.write_ssbh(writer, data_ptr)?;
+            element.ssbh_write(writer, data_ptr)?;
         }
 
         Ok(())
@@ -323,7 +203,7 @@ impl<T: SsbhWrite + binread::BinRead> SsbhWrite for &[T] {
 }
 
 impl<T: binread::BinRead + SsbhWrite + Sized> SsbhWrite for SsbhArray<T> {
-    fn write_ssbh<W: Write + Seek>(
+    fn ssbh_write<W: Write + Seek>(
         &self,
         writer: &mut W,
         data_ptr: &mut u64,
@@ -338,7 +218,7 @@ impl<T: binread::BinRead + SsbhWrite + Sized> SsbhWrite for SsbhArray<T> {
         let pos_after_length = writer.stream_position()?;
         writer.seek(SeekFrom::Start(*data_ptr))?;
 
-        self.elements.as_slice().write_ssbh(writer, data_ptr)?;
+        self.elements.as_slice().ssbh_write(writer, data_ptr)?;
 
         writer.seek(SeekFrom::Start(pos_after_length))?;
 
@@ -357,7 +237,7 @@ impl<T: binread::BinRead + SsbhWrite + Sized> SsbhWrite for SsbhArray<T> {
 }
 
 impl SsbhWrite for NullString {
-    fn write_ssbh<W: Write + Seek>(
+    fn ssbh_write<W: Write + Seek>(
         &self,
         writer: &mut W,
         _data_ptr: &mut u64,
@@ -419,7 +299,7 @@ fn write_rel_ptr_aligned_specialized<
         }
         None => {
             // Null offsets don't increment the data pointer.
-            writer.write_u64::<LittleEndian>(0u64)?;
+            write_u64(writer, 0u64)?;
             Ok(())
         }
     }
@@ -431,21 +311,21 @@ fn write_rel_ptr_aligned<W: Write + Seek, T: SsbhWrite>(
     data_ptr: &mut u64,
     alignment: u64,
 ) -> std::io::Result<()> {
-    write_rel_ptr_aligned_specialized(writer, data, data_ptr, alignment, T::write_ssbh)?;
+    write_rel_ptr_aligned_specialized(writer, data, data_ptr, alignment, T::ssbh_write)?;
     Ok(())
 }
 
 fn write_ssbh_header<W: Write + Seek>(writer: &mut W, magic: &[u8; 4]) -> std::io::Result<()> {
     // Hardcode the header because this is shared for all SSBH formats.
     writer.write_all(b"HBSS")?;
-    writer.write_u64::<LittleEndian>(64)?;
-    writer.write_u32::<LittleEndian>(0)?;
+    write_u64(writer, 64)?;
+    write_u32(writer, 0)?;
     writer.write_all(magic)?;
     Ok(())
 }
 
 impl<const N: usize> SsbhWrite for CString<N> {
-    fn write_ssbh<W: std::io::Write + std::io::Seek>(
+    fn ssbh_write<W: std::io::Write + std::io::Seek>(
         &self,
         writer: &mut W,
         _data_ptr: &mut u64,
@@ -470,8 +350,49 @@ impl<const N: usize> SsbhWrite for CString<N> {
     }
 }
 
+impl<T: SsbhWrite + binread::BinRead> SsbhWrite for Ptr64<T> {
+    fn ssbh_write<W: Write + Seek>(
+        &self,
+        writer: &mut W,
+        data_ptr: &mut u64,
+    ) -> std::io::Result<()> {
+        // TODO: This is nearly identical to the relative pointer function.
+        let alignment = self.0.alignment_in_bytes();
+
+        // The data pointer must point past the containing type.
+        let current_pos = writer.stream_position()?;
+        if *data_ptr <= current_pos {
+            *data_ptr = current_pos + self.size_in_bytes();
+        }
+
+        // Calculate the absolute offset.
+        *data_ptr = round_up(*data_ptr, alignment);
+        write_u64(writer, *data_ptr)?;
+
+        // Write the data at the specified offset.
+        let pos_after_offset = writer.stream_position()?;
+        writer.seek(SeekFrom::Start(*data_ptr))?;
+
+        self.0.ssbh_write(writer, data_ptr)?;
+
+        // Point the data pointer past the current write.
+        // Types with relative offsets will already increment the data pointer.
+        let current_pos = writer.stream_position()?;
+        if current_pos > *data_ptr {
+            *data_ptr = round_up(current_pos, alignment);
+        }
+
+        writer.seek(SeekFrom::Start(pos_after_offset))?;
+        Ok(())
+    }
+
+    fn size_in_bytes(&self) -> u64 {
+        8
+    }
+}
+
 impl<T: SsbhWrite + binread::BinRead> SsbhWrite for RelPtr64<T> {
-    fn write_ssbh<W: Write + Seek>(
+    fn ssbh_write<W: Write + Seek>(
         &self,
         writer: &mut W,
         data_ptr: &mut u64,
@@ -493,14 +414,21 @@ impl<T: SsbhWrite + binread::BinRead> SsbhWrite for RelPtr64<T> {
 }
 
 impl<T: SsbhWrite> SsbhWrite for Vec<T> {
-    fn write_ssbh<W: Write + Seek>(
+    fn ssbh_write<W: Write + Seek>(
         &self,
         writer: &mut W,
         data_ptr: &mut u64,
     ) -> std::io::Result<()> {
-        for elem in self.iter() {
-            elem.write_ssbh(writer, data_ptr)?;
+        // The data pointer must point past the containing struct.
+        let current_pos = writer.stream_position()?;
+        if *data_ptr <= current_pos {
+            *data_ptr = current_pos + self.size_in_bytes();
         }
+
+        for elem in self.iter() {
+            elem.ssbh_write(writer, data_ptr)?;
+        }
+
         Ok(())
     }
 
@@ -516,44 +444,27 @@ impl<T: SsbhWrite> SsbhWrite for Vec<T> {
     }
 }
 
-pub fn write_anim<W: Write + Seek>(writer: &mut W, data: &Anim) -> std::io::Result<()> {
-    write_ssbh_header(writer, b"MINA")?;
-
-    let mut data_ptr = writer.stream_position()?;
-
-    // Point past the struct.
-    data_ptr += data.size_in_bytes(); // size of fields
-
-    data.write_ssbh(writer, &mut data_ptr)?;
-
-    // Padding was added for version 2.1 compared to 2.0.
-    if data.major_version == 2 && data.minor_version == 1 {
-        // The newer file revision is aligned to a multiple of 8.
-        let total_size = writer.seek(SeekFrom::End(0))?;
-        let new_size = round_up(total_size, 8);
-        for _ in 0..(new_size - total_size) {
-            writer.write_all(&[0u8])?;
-        }
-    }
-
-    Ok(())
-}
-
-pub fn write_ssbh<W: Write + Seek>(writer: &mut W, data: &SsbhFile) -> std::io::Result<()> {
+pub(crate) fn write_ssbh_header_and_data<W: Write + Seek>(
+    writer: &mut W,
+    data: &SsbhFile,
+) -> std::io::Result<()> {
     match &data {
         SsbhFile::Modl(modl) => write_ssbh_file(writer, modl, b"LDOM"),
         SsbhFile::Skel(skel) => write_ssbh_file(writer, skel, b"LEKS"),
         SsbhFile::Nufx(nufx) => write_ssbh_file(writer, nufx, b"XFUN"),
         SsbhFile::Shdr(shdr) => write_ssbh_file(writer, shdr, b"RDHS"),
         SsbhFile::Matl(matl) => write_ssbh_file(writer, matl, b"LTAM"),
-        SsbhFile::Anim(anim) => write_anim(writer, &anim),
+        SsbhFile::Anim(anim) => write_ssbh_file(writer, anim, b"MINA"),
         SsbhFile::Hlpb(hlpb) => write_ssbh_file(writer, hlpb, b"BPLH"),
         SsbhFile::Mesh(mesh) => write_ssbh_file(writer, mesh, b"HSEM"),
         SsbhFile::Nrpd(nrpd) => write_ssbh_file(writer, nrpd, b"DPRN"),
     }
 }
 
-pub fn write_buffered<W: Write + Seek, F: Fn(&mut Cursor<Vec<u8>>) -> std::io::Result<()>>(
+pub(crate) fn write_buffered<
+    W: Write + Seek,
+    F: Fn(&mut Cursor<Vec<u8>>) -> std::io::Result<()>,
+>(
     writer: &mut W,
     write_data: F,
 ) -> std::io::Result<()> {
@@ -567,7 +478,7 @@ pub fn write_buffered<W: Write + Seek, F: Fn(&mut Cursor<Vec<u8>>) -> std::io::R
 }
 
 // TODO: This can probably just be derived.
-pub fn write_ssbh_file<W: Write + Seek, S: SsbhWrite>(
+pub(crate) fn write_ssbh_file<W: Write + Seek, S: SsbhWrite>(
     writer: &mut W,
     data: &S,
     magic: &[u8; 4],
@@ -578,7 +489,7 @@ pub fn write_ssbh_file<W: Write + Seek, S: SsbhWrite>(
     // Point past the struct.
     data_ptr += data.size_in_bytes(); // size of fields
 
-    data.write_ssbh(writer, &mut data_ptr)?;
+    data.ssbh_write(writer, &mut data_ptr)?;
     Ok(())
 }
 
@@ -603,13 +514,31 @@ mod tests {
         hex::decode(no_whitespace).unwrap()
     }
 
+    #[test]
+    fn write_ptr() {
+        let value = Ptr64(5u64);
+
+        let mut writer = Cursor::new(Vec::new());
+        let mut data_ptr = 0;
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
+
+        assert_eq!(
+            *writer.get_ref(),
+            hex_bytes(
+                "08000000 00000000 
+                 05000000 00000000"
+            )
+        );
+        assert_eq!(16, data_ptr);
+    }
+
     #[test]
     fn write_null_rel_ptr() {
         let value = RelPtr64::<u32>(None);
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(*writer.get_ref(), hex_bytes("00000000 00000000"));
         assert_eq!(8, data_ptr);
@@ -621,7 +550,7 @@ mod tests {
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
             *writer.get_ref(),
@@ -631,15 +560,16 @@ mod tests {
                  07000000"
             )
         );
+        assert_eq!(20, data_ptr);
     }
 
     #[test]
-    fn write_ssbh_string() {
+    fn ssbh_write_string() {
         let value = SsbhString::from("scouter1Shape");
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
             *writer.get_ref(),
@@ -650,12 +580,12 @@ mod tests {
     }
 
     #[test]
-    fn write_ssbh_string_empty() {
+    fn ssbh_write_string_empty() {
         let value = SsbhString::from("");
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(*writer.get_ref(), hex_bytes("08000000 00000000 00000000"));
         // The data pointer should be aligned to 4.
@@ -663,12 +593,12 @@ mod tests {
     }
 
     #[test]
-    fn write_ssbh_string_non_zero_data_ptr() {
+    fn ssbh_write_string_non_zero_data_ptr() {
         let value = SsbhString::from("scouter1Shape");
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 5;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
             *writer.get_ref(),
@@ -679,7 +609,7 @@ mod tests {
     }
 
     #[test]
-    fn write_ssbh_array_ssbh_string() {
+    fn ssbh_write_array_ssbh_string() {
         let value = SsbhArray::new(vec![
             SsbhString::from("leyes_eye_mario_l_col"),
             SsbhString::from("eye_mario_w_nor"),
@@ -687,7 +617,7 @@ mod tests {
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         // Check that the relative offsets point past the array.
         // Check that string data is aligned to 4.
@@ -709,7 +639,7 @@ mod tests {
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         // Null and empty arrays seem to use 0 offset and 0 length.
         assert_eq!(
@@ -725,7 +655,7 @@ mod tests {
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
             *writer.get_ref(),
@@ -734,13 +664,28 @@ mod tests {
         assert_eq!(21, data_ptr);
     }
 
+    #[test]
+    fn write_vec() {
+        let value = vec![1u8, 2u8, 3u8, 4u8, 5u8];
+
+        let mut writer = Cursor::new(Vec::new());
+        let mut data_ptr = 0;
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
+
+        assert_eq!(
+            *writer.get_ref(),
+            hex_bytes("01020304 05")
+        );
+        assert_eq!(5, data_ptr);
+    }
+
     #[test]
     fn write_empty_byte_buffer() {
         let value = SsbhByteBuffer::new(Vec::new());
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         // Null and empty arrays seem to use 0 offset and 0 length.
         assert_eq!(
@@ -751,7 +696,7 @@ mod tests {
     }
 
     #[test]
-    fn write_ssbh_string_tuple() {
+    fn ssbh_write_string_tuple() {
         #[derive(SsbhWrite)]
         struct StringPair {
             item1: SsbhString,
@@ -766,7 +711,7 @@ mod tests {
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         // Check that the pointers don't overlap.
         assert_eq!(
@@ -781,12 +726,12 @@ mod tests {
     }
 
     #[test]
-    fn write_ssbh_string8() {
+    fn ssbh_write_string8() {
         let value = SsbhString8::from("BlendState0");
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
             *writer.get_ref(),
@@ -797,12 +742,12 @@ mod tests {
     }
 
     #[test]
-    fn write_ssbh_string8_empty() {
+    fn ssbh_write_string8_empty() {
         let value = SsbhString8::from("");
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
             *writer.get_ref(),
@@ -813,12 +758,12 @@ mod tests {
     }
 
     #[test]
-    fn write_ssbh_string8_non_zero_data_ptr() {
+    fn ssbh_write_string8_non_zero_data_ptr() {
         let value = SsbhString8::from("BlendState0");
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 5;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
             *writer.get_ref(),
@@ -828,34 +773,17 @@ mod tests {
         assert_eq!(24, data_ptr);
     }
 
-    #[derive(BinRead, PartialEq, Debug)]
+    #[derive(BinRead, PartialEq, Debug, SsbhWrite)]
     #[br(import(data_type: u64))]
     pub enum TestData {
-        #[br(pre_assert(data_type == 01u64))]
+        #[br(pre_assert(data_type == 1u64))]
         Float(f32),
-        #[br(pre_assert(data_type == 02u64))]
+        #[br(pre_assert(data_type == 2u64))]
         Unsigned(u32),
     }
 
-    impl SsbhWrite for TestData {
-        fn write_ssbh<W: Write + Seek>(
-            &self,
-            writer: &mut W,
-            data_ptr: &mut u64,
-        ) -> std::io::Result<()> {
-            match self {
-                TestData::Float(f) => f.write_ssbh(writer, data_ptr),
-                TestData::Unsigned(u) => u.write_ssbh(writer, data_ptr),
-            }
-        }
-
-        fn size_in_bytes(&self) -> u64 {
-            todo!()
-        }
-    }
-
     #[test]
-    fn write_ssbh_enum_float() {
+    fn ssbh_write_enum_float() {
         let value = SsbhEnum64::<TestData> {
             data: RelPtr64::new(TestData::Float(1.0f32)),
             data_type: 1u64,
@@ -863,7 +791,7 @@ mod tests {
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
             *writer.get_ref(),
@@ -872,7 +800,7 @@ mod tests {
     }
 
     #[test]
-    fn write_ssbh_enum_unsigned() {
+    fn ssbh_write_enum_unsigned() {
         let value = SsbhEnum64::<TestData> {
             data: RelPtr64::new(TestData::Unsigned(5u32)),
             data_type: 2u64,
@@ -880,7 +808,7 @@ mod tests {
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
-        value.write_ssbh(&mut writer, &mut data_ptr).unwrap();
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
             *writer.get_ref(),
