@@ -10,7 +10,7 @@ use binread::BinRead;
 
 #[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, SsbhWrite)]
-pub struct AnimTrack {
+pub struct AnimTrackV2 {
     pub name: SsbhString,
     pub flags: TrackFlags,
     pub frame_count: u32,
@@ -23,7 +23,7 @@ pub struct AnimTrack {
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct AnimNode {
     pub name: SsbhString,
-    pub tracks: SsbhArray<AnimTrack>,
+    pub tracks: SsbhArray<AnimTrackV2>,
 }
 
 #[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
@@ -34,27 +34,87 @@ pub struct AnimGroup {
 }
 
 /// Skeletal and material animation.
-/// Compatible with file version 2.0 and 2.1.
+/// Compatible with file version 1.2, 2.0, and 2.1.
 #[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct Anim {
     pub major_version: u16,
     pub minor_version: u16,
+    #[br(args(major_version, minor_version))]
+    pub header: AnimHeader,
+}
+
+#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[derive(BinRead, Debug, SsbhWrite)]
+#[br(import(major_version: u16, minor_version: u16))]
+pub enum AnimHeader {
+    #[br(pre_assert(major_version == 1 && minor_version == 2))]
+    HeaderV1(AnimHeaderV12),
+
+    #[br(pre_assert(major_version == 2 && minor_version == 0))]
+    HeaderV20(AnimHeaderV20),
+
+    #[br(pre_assert(major_version == 2 && minor_version == 1))]
+    HeaderV21(AnimHeaderV21)
+}
+
+#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[derive(BinRead, Debug, SsbhWrite)]
+pub struct AnimHeaderV12 {
+    pub name: SsbhString,
+    pub unk1: u32,
+    /// The index of the last frame in the animation,
+    /// which is calculated as `(frame_count - 1) as f32`.   
     pub final_frame_index: f32,
-    pub unk1: u16, // always 1
-    pub unk2: u16, // always 3
+    pub unk2: u64,
+    pub tracks: SsbhArray<AnimTrackV1>,
+    pub buffers: SsbhArray<SsbhByteBuffer>
+}
+
+#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[derive(BinRead, Debug, SsbhWrite)]
+pub struct AnimTrackV1 {
+    pub name: SsbhString,
+    pub track_type: u64,
+    pub properties: SsbhArray<Property>
+}
+
+#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[derive(BinRead, Debug, SsbhWrite)]
+pub struct Property {
+    pub name: SsbhString,
+    /// The index of the corresponding buffer in [buffers](struct.AnimHeaderV12.html#structfield.buffers).
+    pub buffer_index: u64,
+}
+
+#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[derive(BinRead, Debug, SsbhWrite)]
+pub struct AnimHeaderV20 {
+    /// The index of the last frame in the animation,
+    /// which is calculated as `(frame_count - 1) as f32`.
+    pub final_frame_index: f32,
+    // TODO: Is this some other version?
+    pub unk1: u16, // always 1?
+    pub unk2: u16, // always 3?
     pub name: SsbhString,
     pub animations: SsbhArray<AnimGroup>,
     pub buffer: SsbhByteBuffer,
+}
 
-    // TODO: Find a cleaner way to handle serializing.
-    #[cfg_attr(
-        feature = "derive_serde",
-        serde(skip_serializing_if = "Option::is_none")
-    )]
-    #[cfg_attr(feature = "derive_serde", serde(default))]
-    #[br(if(major_version == 2 && minor_version == 1))]
-    pub unk_data: Option<UnkData>,
+#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[derive(BinRead, Debug, SsbhWrite)]
+#[ssbhwrite(align_after = 8)]
+pub struct AnimHeaderV21 {
+    /// The index of the last frame in the animation,
+    /// which is calculated as `(frame_count - 1) as f32`.
+    pub final_frame_index: f32,
+    // TODO: Is this some other version?
+    pub unk1: u16, // always 1?
+    pub unk2: u16, // always 3?
+    pub name: SsbhString,
+    pub animations: SsbhArray<AnimGroup>,
+    pub buffer: SsbhByteBuffer,
+    pub unk_data: UnkData,
 }
 
 #[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
