@@ -14,7 +14,7 @@ pub struct ModlEntry {
     pub mesh_object_name: SsbhString,
 
     /// The `sub_index` of the [MeshObject](crate::formats::mesh::MeshObject).
-    pub mesh_object_sub_index: i64,
+    pub mesh_object_sub_index: u64,
 
     /// The `material_label` of the [MatlEntry](crate::formats::matl::MatlEntry).
     pub material_label: SsbhString,
@@ -28,7 +28,7 @@ pub struct Modl {
     pub major_version: u16,
     pub minor_version: u16,
     /// The name of the model such as "model".
-    pub model_name: SsbhString,
+    pub model_name: SsbhString, // TODO: this might be the source file used to generate the .numdlb
     /// The name of the associated [Skel](crate::formats::skel::Skel) file such as "model.nusktb".
     pub skeleton_file_name: SsbhString,
     /// The names of the associated [Matl](crate::formats::matl::Matl) files.
